//------------------------------------------------------------------
/*
	EEPROM.c

	Routines to access the EERPOM on the timer module.
	
	The EEPROM is: 32k x 8, serial SPI bus, 25LC128, Microchip.

*/
//------------------------------------------------------------------
// Constants and global variables
#include <constants.h>
#include <externals.h>
// C libraries
#include <propeller.h>
#include <stdio.h>
#include <asio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

//------------------------------------------------------------------
// Protoypes of external routines
int Get_String(char *str, int len);					// Serial.c
char Echo_Key(void);												// Serial.c
char SPI_Byte(char send_char);							// SPI.c
void Initialise_Parameters(void);						// Utilities.c
void Print_Line(int length);								// Utilities.c

//------------------------------------------------------------------ START
void EEPROM_write(int address, int value)
/*
	Writes a value to the EEPROM on the timer module using the SPI bus.
	
	INPUTS:		int(16) address = EEPROM location where data is to be saved
						int(8)  value		= value to be written to address
	OUTPUTS:	None
*/
	{
	int addMSB, addLSB, bytein, byteout;				// bytein and byteout are dummy variables for the SPI_Byte() function
	
	addMSB = address/256;												// address needs to be split into bytes for SPI transfer
	addLSB = address - addMSB * 256;
	
	CLR(OUTA, CSNEEPROM);												// Select EEPROM chip on the timer module
	bytein = SPI_Byte(0x06);										// Send "Enable Write" command to EEPROM
	SET(OUTA, CSNEEPROM);												// EEPROM needs to be deselected for "Enable Write" to take effect
	msleep(1);																	// Some time
	CLR(OUTA, CSNEEPROM);												// Select EEPROM again
	bytein = SPI_Byte(0x02);										// Send "Write" command to the EEPROM
	bytein = SPI_Byte(addMSB);									// Send write address MSB
	bytein = SPI_Byte(addLSB);									// Send write address LSB
	bytein = SPI_Byte(value);				  					// Send data to be stored in EEPROM	
	SET(OUTA, CSNEEPROM);												// Deselect the EEPROM
	
	for(byteout = 0; byteout < 1000; byteout++)	// Wait for write cycle to finish by checking the flag in the EEPROM
		{
		CLR(OUTA, CSNEEPROM);											// Select the EEPROM
		bytein = SPI_Byte(0x05);									// Send "Read Status" command
		bytein = SPI_Byte(0x00);									// Fetch the Statur register in the EEPROM
		if((bytein & 0x01) == 0x00)								// Check  write finished flag
			byteout = 1000;													// Kill the loop if writing is finished
		SET(OUTA, CSNEEPROM);											// Deselect the EEPROM
		}
	}	
//------------------------------------------------------------------	
int EEPROM_read(int address)
/*
	Reads a value from the EEPROM on the timer module using the SPI bus.
	
	INPUTS:		int(16)	address = EEPROM location that is to be read
	RETURNS:	int(8)  value		= the value read from the EEPROM
*/
	{
	int addMSB, addLSB, bytein;									// bytein is a dummy variable for the SPI_Byte() function
	
	addMSB = address/256;												// address needs to be split into bytes for SPI transfer
	addLSB = address - addMSB * 256;
	
	CLR(OUTA, CSNEEPROM);												// Select EEPROM
	bytein = SPI_Byte(0x03);										// Send "Read" instruction to the EEPROM
	bytein = SPI_Byte(addMSB);									// Send the address MSB
	bytein = SPI_Byte(addLSB);									// Send the address LSB
	bytein = SPI_Byte(0x00);				  					// Fetch the value at the address location		
	SET(OUTA, CSNEEPROM);												// Deselect the EEPROM
	return(bytein);															// Return the value
	}
//------------------------------------------------------------------
void Get_EEString(char *str, int eeaddress)
/*
 Function to read a character string from the EEPROM.
 Character strings are stored in blocks of 16 bytes.
 
 INPUTS:	pointer	*str			= Pointer to storage location for the string to be read
 					int(16)	eeaddress	= Starting address for reading the EEPROM
 RETURNS:	None
*/
	{
	int j;
	
	eeaddress = eeaddress * 16;						// Actual EEPROM starting address is 16 x address
	for(j=0; j<16; j++)										// Fetch 16 characters
		{
		str[j] = EEPROM_read(eeaddress);		// Fetch one character
		eeaddress++;												// Point to the next location
		}
	}
//------------------------------------------------------------------	
int Get_EEPROM_Int(int eeaddress)
/*
	Reads an integer value from the EEPROM.
	
	INPUTS:		int(16)	eeaddress = the address of the integer to be fetched
	RETURNS:	int(32) value			= integer value found at address
*/
	{
	char invalue[16];											// Reserve space for the character string from the EEPROM
	Get_EEString(invalue, eeaddress);			// Fetch the characters from the EEPROM (max 16)
	return(atoi(invalue));								// Convert the characters into an integer then return it
	}
//------------------------------------------------------------------	
float Get_EEPROM_Float(int eeaddress)
/*
	Reads a floating point value from the EEPROM.
		
	INPUTS:		int(16)		eeaddress = the address of the floating point number to be fetched
	RETURNS:	float(32) value			= floating point value found at address
*/
	{
	char invalue[16];											// Reserve space for the character string from the EEPROM
	Get_EEString(invalue, eeaddress);			// Fetch the characters from the EEPROM (max 16)
	return(atof(invalue));								// Convert the characters into a floating point number then return it
	}
//------------------------------------------------------------------
void EEPROM_Line(int line)
/*
	Displays a line showing EEPROM address and EEPROM data as follows:
		address:data
	
	INPUTS:		int line	=	the EEPROM address of the data to be displayed
	RETURNS:	None
*/
	{
	int j;
	char invalue;
	
	printf("\r%3d:", line);										// Display the address
	line = line * 16;													// The actuall EEPROM address is 16 x address
	for(j=0; j<16; j++)												// Fetch and display all 16 characters in the line
		{
		invalue = EEPROM_read(line);						// Fetch one character
		if((invalue > 21) & (invalue < 127))		// Make sure that it is a printable character
			putchar(invalue);											// If it is then print it otherwise ignore it
		line++;																	// Point to the next character
		}
	}
//------------------------------------------------------------------
void EEPROM_Display(void)
/*
	Displays all lines of data from the EEPROM. Used as a heading for the EEPROM editor.
	
	INPUTS:		None
	RETURNS:	None
*/
	{
	int i;
	
	printf("\rEEPROM Editor\r");			// Show EEPROM editor heading
	Print_Line(13);										// With a line under it	
	for(i = 0; i < 28; i++)						// Show all used EEPROM locations
		EEPROM_Line(i);									// One line at a time
	printf("\r\rPress Q to quit\r");		// Show EEPROM editor footer
	}
//------------------------------------------------------------------
void EEPROM_Editor(void)
/*
	Routine to edit values in the EEPROM.
	The routine will repeat until an 'x' is entered.
	If no data is entered (either address or value) then the routine starts again.
	
	INPUTS:		None
	RETURNS:	None
*/
	{
	int inkey, address, charpointer, lastchar;
	char inaddress[4], invalue[16];
	float value;

	while(inkey != 'q')															// Keep editing until the user enters a 'q'
		{
		memset(inaddress,0,4);												// Clear space for the EEPROM address 
		memset(invalue,0,16);													// Clear space for the EEPROM value
		EEPROM_Display();															// Show the EEPROM editor heading
		
																									// Get the address first
		printf("address: ");													// Prompt the user to enter an address
		charpointer = 0;															// Clear the character pointer
		inkey = Echo_Key();														// Fetch and echo the first key press
		if((inkey != 'q') & (inkey != 13))						// If it is 'q' then quit, <CR> abort, otherwise carry on
			{
			while((inkey != 13) & (charpointer < 4))		// Keep getting keys until either <CR> or too many characters
			{																															
			inaddress[charpointer] = inkey;										// Store the key press
			inkey = Echo_Key();													// Fetch and echo the next key press
			charpointer++;															// Point to the next storage location
			}		
			address = atoi(inaddress);									// Convert the address into an integer for later use
		
																									// Get the characters to be saved at the EEPROM address
			printf("value: ");													// Prompt the user to enter a value
			charpointer = 0;														// Clear the character pointer
			inkey = Echo_Key();													// Fetch and echo the first key press
			if((inkey != 'q') & (inkey != 13))					// If it is 'q' then quit, <CR> abort, otherwise carry on
				{
				while((inkey != 13) & (charpointer < 16))	// Keep getting keys until either <CR> or too many characters
					{																															
					invalue[charpointer] = inkey;								// Store the key press
					inkey = Echo_Key();											// Fetch and echo the next key press
					charpointer++;													// Point to the next storage location
					}
					
				printf("Save(y/n)?");											// Prompt the user to save the data
				inkey = Echo_Key();												// Fetch the response
				if(inkey == 'y')													// If it's yes then save the data in the address
					{
					lastchar = charpointer;									// Check how many valid characters need to be saved
					address = address * 16;									// Point to the starting address
					for(charpointer = 0; charpointer < 16; charpointer++)	// Save 16 characters
						{
						if(charpointer < lastchar)						// Save the entered characters up to the end of the entered string
							EEPROM_write(address, invalue[charpointer]);			// Save to EEPROM
						else
							EEPROM_write(address, 0);						// Pad nul characters after the end of the entered string
						address++;														// Point to the next address
						}
					Initialise_Parameters();								// Load the new values into local RAM after saving
					}
				}
			}
		}
	}
//------------------------------------------------------------------ END