//------------------------------------------------------------------
/*
	SPI.c

	Routines dealing with communications between the Propeller processor and the DS00VQ100 chip.

*/
//------------------------------------------------------------------
// Constants and global variables
#include <constants.h>
#include <externals.h>
// C libraries
#include <propeller.h>
#include <stdio.h>
#include <asio.h>

//------------------------------------------------------------------
// Protoypes of external routines

//------------------------------------------------------------------ START
char SPI_Byte(char send_char)
/*
	Function to transmit and receive bytes over the SPI bus.
	Transmit and receive occur simultaneously.
	Chip select is handled elsewhere.
	
	INPUTS:		char	send_char	= character to send out on the SPI bus
	RETURNS:	char	value			= character read from the SPI bus
*/
	{
	int get_char = 0;
	int out_mask = 0x80;									// Used as a bit pointer
	int i = 0;
	
	for(i=0; i<=7; i++)										// Send and receive 8 bits
		{
		get_char = get_char << 1;						// Rotate input buffer ready to receive a bit
		
		if((send_char & out_mask) != 0)			// Check bit on send buffer
			SET(OUTA, MOSI);									// Output a 1
		else
			CLR(OUTA, MOSI);									// Or a 0

		SET(OUTA, SPI_CLOCK);								// Provide a rising clock edge
		if((INA & IN_MASK) != 0)						// If there is an incomming 1 then
			get_char++;												// Add it to the LS bit of the receive buffer
		CLR(OUTA, SPI_CLOCK);								// Provide a falling clock edge
				
		out_mask = out_mask >> 1;						// Rotate the outpur buffer ready to send the next bit
		}
	return get_char;											// Return the input buffer
	}	
//------------------------------------------------------------------
int SPI (void)
/*
	Checks for new data ready from DS00VQ100 chip
	OR a request for an asynchronous data load
	THEN executes a 14 byte transfer on the SPI bus.
	
	Converts 8 bit MSB/LSB data into 16 bit results where necessary.
	Updates other parameters for later processing.

	Inputs:		None
 	Returns: 	int NOT_READY if no new values are ready.
						int READY if new values are available for further processing.
*/
	{
	if(((INA & DATA_READY_MASK) == READY) | (SPI_Footer == ASYNC_FOOTER))	// Check for new data ready or asynchronous transfer request
    {
    Index++;																			// Update index associated with the next result
		Index = Index & 0x03;													// Limit index to a count range of 0 - 3

		SET(OUTA, CS);																// Select the DS00VQ100 chip
		msleep(1);																		// Provide a short pause before starting SPI transfer
    Firmware = SPI_Byte(SPI_HEADER);							// Start sending and receiving bytes over the SPI bus
    Zero_Rear_MSB = SPI_Byte(0);									// Send "Configure" as a fixed value of 0
    Zero_Rear_LSB = SPI_Byte(0);									// Send "UnusedB" as a fixed value of 0
    Zero_Front_MSB = SPI_Byte(min_width);
    Zero_Front_LSB = SPI_Byte(APD_Voltage);
    Signal_Rear_MSB = SPI_Byte(Laser_Voltage);
    Signal_Rear_LSB = SPI_Byte(Index);
		Signal_Front_MSB = SPI_Byte(min_width);				// Send "Zero_Width" as min_width - the same as Signal_Width
    Signal_Front_LSB = SPI_Byte(Test_Port);
    Noise_MSB = SPI_Byte(Measuring_Range);
    Noise_LSB = SPI_Byte(Main_Xtal);
    Span_MSB = SPI_Byte(Second_Xtal); 
    Span_LSB = SPI_Byte(Amplifier_Gain);  
    Status = SPI_Byte(SPI_Footer); 								// Either synchronous or asychronous load of data into the DS00VQ100 chip
		msleep(1); 																		// Provide a short pause before ending SPI transfer
		CLR(OUTA, CS); 																// Deselect the DS00VQ100 chip

    SPI_Footer = SYNC_FOOTER;    									// Always return to synchronous SPI write for next data transfer
		switch(Measuring_Range)												// Keep the measuring range updated for later calculations
			{
			case 1:
				range = RANGE1;
			break;
			case 2:
				range = RANGE2;
			break;
			case 3:
				range = RANGE3;
			break;
			case 4:
				range = RANGE4;
			break;		
			default:
				range = RANGE0;
			break;
			}
		
		if ((Status & TIME_DIR_MASK) == FORWARD)			// Update local time expansion direction flag for later use
			direction = FORWARD;												
		else
			direction = REVERSE;			
																									// Convert 8 bit values to 16 bit values where neccesary
		noise_int	= Noise_MSB * 256 + Noise_LSB;
		span_int = Span_MSB * 256 + Span_LSB;
																									// Some 8 bit to 16 bitconversions depend on the direction of time expansion
		if (direction == FORWARD)											// Forward time expansion
			{
			zero_front_int = Zero_Front_MSB * 256 + Zero_Front_LSB;
			zero_rear_int = Zero_Rear_MSB * 256 + Zero_Rear_LSB;		
			signal_front_int = Signal_Front_MSB * 256 + Signal_Front_LSB;
			signal_rear_int = Signal_Rear_MSB * 256 + Signal_Rear_LSB;
			}
		else																					// Reverse time expansion
			{
			zero_rear_int = span_int - (Zero_Front_MSB * 256 + Zero_Front_LSB);
			zero_front_int = span_int - (Zero_Rear_MSB * 256 + Zero_Rear_LSB);		
			signal_rear_int = span_int - (Signal_Front_MSB * 256 + Signal_Front_LSB);
			signal_front_int = span_int - (Signal_Rear_MSB * 256 + Signal_Rear_LSB);	
			}
			
		signal_width_int = signal_rear_int - signal_front_int;		
			
		return (READY); 
    } 
	else	
		return (NOT_READY);
	}
	
//------------------------------------------------------------------ END