//------------------------------------------------------------------
/*
	Serial.c

	Routines dealing with communications through the USB port.
	See description of commands and display options in the Read_Me.txt document.

	Designed to work with the Parallax Serial Terminal.

*/
//------------------------------------------------------------------
// Constants and global variables
#include <constants.h>
#include <externals.h>
// C libraries
#include <propeller.h>
#include <stdio.h>
#include <asio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

//------------------------------------------------------------------
// Protoypes of external routines
void EEPROM_Editor(void);						// EEPROM.c
void Initialise_Parameters(void);		// Utilities.c
float Rescm(void);									// Utilities.c

//------------------------------------------------------------------ START
char Echo_Key(void)
/*
	A function to get a character from the USB port, echo it and set it to lower case.
	
	INPUTS:		None
	RETURNS:	char value = any lower case character
*/
	{
	int inkey;	
	inkey = getchar();				// Fetch character
	putchar(inkey);						// Echo
	return(tolower(inkey));		// Drop to lower case and return
	}
//------------------------------------------------------------------		
void Print_Line(int length)
/*
	A display formatting function that prints a line using the '-' character.
	
	INPUTS:		int length	=	length of line to be printed
	RETURNS:	None
*/
	{
	while(length > 0)				// Repeat until line is printed
		{
		putchar('-');					// -
		length--;							// Keep going
		}
	putchar('\r');					// Put a <CR> at the end of the line
	}
//------------------------------------------------------------------
int Get_String(char *str, int len)
/*
 A function to input a string of characters whilst echoing them on the console.
 
 INPUTS:	pointer *str				= pointer to the storage area of the string to be entered
					int 		len					= the maximum expacted length of the string
 RETURNS:	int 		charpointer	= the number of characters entered
*/
	{
	int inkey = 0;
	int charpointer = 0;

	inkey = Echo_Key();													// Fetch the first key
	while((inkey != 13) & (charpointer < len))	// Keep getting keys until either <CR> or too many characters
		{																															
		str[charpointer] = inkey;									// Store the key press
		inkey = Echo_Key();												// Fetch and echo the next key press
		charpointer++;														// Point to the next storage location
		}
	return(charpointer);												// Return the number of characters entered
	}
//------------------------------------------------------------------		
void Settings_Display(void)
/*
	Displays the current settings as follows:

		System Settings
	-------------------
	Amplifier Gain  8					= the gain of the post-amplifier
	APD Voltage     138				= the bias voltage on the APD
	Laser Voltage   20				= a value used to start the laser firing
	Measuring Range 0					= a value that switches between the preset measuring ranges
	Test Port       2					= a value that indicates which test signals are output on the test pins
	Crystal         242				=	the control value of the crystal that determines the resolution
	-------------------
	Set Noise       50				= setpoint value for automatic APD noise control
	Average on/off  0					=	indicates whether averaging is on (1) or off (0), 16 results are averaged
	Set Resolution  +9				= setpoint value for automatic resolution control in centimeters
	Set Zero Datum  -100			= an offset value for the zero measuring datum in centimeters

	INPUTS:		None
	RETURNS:	None
*/
  {	
	printf("\r  System Settings\r");										// Heading
	Print_Line(19);																			// Separator
																											// DS00VQ100 settings
  printf("Amplifier Gain  %d\rAPD Voltage     %d\rLaser Voltage   %d\r", Amplifier_Gain, APD_Voltage, Laser_Voltage);
	printf("Measuring Range %d\rTest Port       %d\rCrystal         %d\r", Measuring_Range, Test_Port, Second_Xtal);
	Print_Line(19);																			// Separator
																											// Software settings
	printf("Set Noise       %d\rAverage on/off  %d\rSet Resolution  ", con_noise, average_flag);
	
	if(con_resolution < 0)															// Bypass formatting bug in printf function
		printf("%d\r", con_resolution);
	else
		printf("+%d\r", con_resolution);
	if(zero_offset < 0)
		printf("Set Zero Datum  %d\r", zero_offset);
	else		
		printf("Set Zero Datum  +%d\r", zero_offset);
	}
//------------------------------------------------------------------
void Raw_Display(void)
/*
	Displays the raw data that comes from the DS00VQ100 chip as follows:

	ZF:  569    ZR:  904    | SF:  900    SR: 1789    | AN:   31    | SP: 2471    | ST:11000001

		ZF = counts to the front of the zero pulse
		ZR = counts to the rear of the zero pulse
		SF = counts to the front of the return signal
		SR = counts to the rear of the return signal
		AN = an indication of the noise coming from the APD detector in counts
		SP = the number of counts in the measuring range (span)
		ST = eight bit status flags from the Status register
				B7 		= valid zero pulse
				B6 		= valid return signal
				B5 		= time expansion direction (0=forward, 1=reverse)
				B4 		= 0
				B3..0 = an index value used to identify specific results
				
	INPUTS:		None
	RETURNS:	None
*/
  {
	int j;
	int mask = 0x80;
	
	if(direction == FORWARD)				// Modify the Span display to indicate the time expansion direction
		j = span_int;									// Forward time expansion results in a positive Span value
	else
		j = 0 - span_int;							// Reverse time expansion results in a negative Span value
			
	printf("ZF:%5d    ZR:%5d    | SF:%5d    SR:%5d    | AN:%5d    | SP:%5d    | ST:", zero_front_int, zero_rear_int, signal_front_int, signal_rear_int, noise_int, j, Status);
	
																	// Unpack the Status register to print as a binary number
	for(j=0; j<8; j++)							// ANSI C does not have a binary print function
		{ 														// There is insufficient code space to make this a seperate function
		if((Status & mask) != 0)
			putchar('1');
		else
			putchar('0');
		mask = mask >> 1;
		}
		
	putchar('\r'); 									// Prepare for the next result
  }
//------------------------------------------------------------------ 
void Distances_Display(void)
/*
	Shows the data from the DS00VQ100 chip converted into distance units:
 
	DS:  2.04    | ZF:  5.28    | SF:  8.50    SW:  8.25    | RES:  8.3

		DS = distance to the target in meters
		ZF = distance to the front of the zero pulse in meters
		SF = distance to the front of the return signal in meters
		SW = the width of the return signal in meters - this is an indication of the signal strength
		RES = the resolution of the results in centimeters
		
	INPUTS:		None
	RETURNS:	None
*/
  {																						// Check the Status register to see if signal is valid
	if((Status & SIGNAL_VALID_MASK) != 0)				// Signal is valid - wide enough for an accurate measurement
		printf("DS:%#6.2f    | ZF:%#6.2f    | SF:%#6.2f    SW:%#6.2f", distance, zero_front, signal_front, signal_width);
	else																				// Signal is invalid - either too small or lost altogether
		printf("DS:---.--    | ZF:%#6.2f    | SF:---.--    SW:---.--", zero_front);
		
	printf("    | RES:%#5.1f\r", Rescm());			// Show the resolution in centimeters with sign
  }   
//------------------------------------------------------------------
void Fast_Display(void)
/*
	This is the default display on power up. It shows the distance in meters:
 		2.96
		
	INPUTS:		None
	RETURNS:	None
*/
  {
	if((Status & SIGNAL_VALID_MASK) != 0)		// Check the Status register to see if signal is valid
		printf("%#6.2f\r", distance);					// Signal is valid - wide enough for an accurate measurement
	else
		printf("---.--\r");  									// Signal is invalid - either too small or lost altogether
  }
//------------------------------------------------------------------ 
void Control_Display(void)
/*
	Shows information relating to the APD noise and time expansion control loops:
 
	SAN:   50    AV:  137    AN:   77    | CRES:    9    SXV:  242    MSW:   12    RES:  8.3

		SAN 	= the setpoint for the APD noise in counts
		AV 		= the APD bias voltage used to achieve set noise value
		AN 		= the actual APD noise measured by the DS00VQ100 chip
		CRES 	= the setpoint for the resolution in centimeters
		SXV 	= the crystal control voltage used to achieve the set resolution
		MSW 	= the count value for the minimum valid signal width at the present resolution
		RES 	= the actual resolution in centimeters as measured by the DS00VQ100 chip
		
	INPUTS:		None
	RETURNS:	None
*/
  {
	printf("SAN:%5d    AV:%5d    AN:%5d    | CRES:%5d    SXV:%5d    MSW:%5d    RES:%#5.1f\r", con_noise, APD_Voltage, noise_int, con_resolution, Second_Xtal, min_width, Rescm());
  }
//------------------------------------------------------------------
void Enter_Command(void)
/* 
	The user can change settings in the LRF using commands. Commands are not case sensitive.
	Command entry has been initiated by pressing the <SPACE> key.

  	Command        		Meaning                   	Description
  	-------        		-------                   	-----------
		A<value> ....... Set Amplifier Gain ..... Sets the post-amplifier gain [0-15]. 
		B<value> ....... Set APD Voltage ........ Sets the APD bias voltage [0-255].
		C<value> ....... Set Measuring Range .... 0 = 23.4375m, 1 = 46.875m, 2 = 93.75m, 3 = 187.5m, 4 = 375.0m.
		D<value> ....... Select Test Port ....... Selects test signals.
		E<value> ....... Set Xtal Frequency ..... Sets the frequency of the secondary oscillator [0-255].
		F<value> ....... Control APD Noise ...... Sets the acceptible noise level on the APD for automatic control [0-32000].
		G<value> ....... Control Resolution ..... Sets the resolution in centimeters [-9 to +9]. Affects the measuring update rate.
		H<value> ....... Set Zero Datum ......... Sets the zero datum point in centimeters.
		I<value> ....... Set Buzzer Range ....... Sets the distance in centimeters at which the buzzer turns on.
		
	INPUTS:		None
	RETURNS:	None
*/
	{
	char incommand;
	char invalue[8];
	int value = 0;

	memset(invalue,0,8);												// Clear value string for later use
	printf("command: ");												// Ask the user for a command
	incommand = Echo_Key();											// Get the command
	if((incommand >= 'a') & (incommand < 'j'))	// Check for a valid command
		{																					// Fetch a value if the command is valid
		printf(" value: ");												// Ask the user for a value
			if(Get_String(invalue, 8) > 0)					// If a value is entered then continue
				{
				value = atoi(invalue);								// Convert the value string into a number
				switch(incommand)											// Choose the command to complete
					{
					case 'a':
						Amplifier_Gain = value;
					break;
					case 'b':
						APD_Voltage = value;
					break;
					case 'c':
						Measuring_Range = value;
					break;
					case 'd':
						Test_Port = value;
					break;
					case 'e':
						Second_Xtal = value;
					break;
					case 'f':
						con_noise = value;
					break;
					case 'g':
						con_resolution = value;
					break;
					case 'h':
						zero_offset = value;
					break;
					case 'i':		// Reserved for future use
					break;
					default:
					break;
					}
				}
			else
				putchar('?');													// If the value is not valid then print a ?		
		}
	else
		putchar('?');															// If the command is not valid then print a ?
	}
//------------------------------------------------------------------  
void User_Action()
/*
	Certain key presses cause user actions to take place immediately. These are not case sensitive.

		Key Press				Action													Description
		---------				-------													-----------
		<SPACE> ... Enter New Setting ............. Opens command editor to enter new values
		A ......... Toggle Averaging on/off ....... Switch between direct distance or 16 distance averages.
  	C ......... Display Control Data .......... Displays software control loop data.
		D ......... Display Distance Data ......... Displays the data as distances.
		E ......... EEPROM Editor ................. Enter the EEPROM editor.
  	F ......... Fast Distance Display ......... Displays the final distance in meters only. 
		I ......... Immediate Load ................ Forces an immediate transfer of new settings to the DS00VQ100 chip.      
  	R ......... Display raw data .............. Displays data directly from the DS00VQ100 chip.
		S ......... Static Settings ............... Displays the current settings. 
		X ......... Stop Laser .................... Switches off the laser.
		Z ......... Start Laser ................... Switches on the laser. 
		
	INPUTS:		None
	RETURNS:	None
*/
  {
  char inkey;
	
  inkey = global_key;  										// Fetch key press from another cog 
	inkey = tolower(inkey);									// Switch to lower case
	putchar(inkey);													// Echo the key on the USB port
	switch(inkey)														// Decode key press
		{
		case ' ':															// The space character takes the user to the settings editor
			Enter_Command();
			if(h2m_mode == 's')									// If the static display is selected then refresh the screen
				Settings_Display();
		break;
		case 'a':															// Toggle the averaging flag
			average_flag = !average_flag;	
		break;
		case 'c':
			h2m_mode = 'c';											// Switch to control display
		break;
		case 'd':
			h2m_mode = 'd';											// Switch to distances display
		break;
		case 'e':
			EEPROM_Editor();										// Take the user to the EEPROM editor
		break;
		case 'f':
			h2m_mode = 'f';											// Switch to fast display mode where only the distance is shown
		break;
		case 'i':
			SPI_Footer = 0x77;									// Immediately load any new settings
		break;
		case 'r':
			h2m_mode = 'r';											// Switch to raw data display
		break;
		case 's':
			h2m_mode = 's';											// Switch to static settings display
			Settings_Display();									// and refresh the screen
		break;
		case 'x':															
      Laser_Voltage = 0;									// Turn off the laser
      SPI_Footer = 0x77;									// and immeditaley execute
    break;
    case 'z':
			Laser_Voltage = 20;									// Turn on the laser
			SPI_Footer = 0x77;									// and immeditaley execute
    break;
		default:
			putchar('?');												// Invalid action
		break;
		}
	}

//------------------------------------------------------------------ END
