//------------------------------------------------------------------
/*
	Utilities.c
	
	Miscellaneous functions used by other routines.
	
*/
//------------------------------------------------------------------
// Constants and global variables
#include <constants.h>
#include <externals.h>
// C libraries
#include <propeller.h>
#include <stdio.h>
#include <asio.h>
#include <math.h>
#include <float.h>
#include <ctype.h>

//------------------------------------------------------------------
// Protoypes of external routines
int Get_EEPROM_Int(int eeaddress);			// EEPROM.c
float Get_EEPROM_Float(int eeaddress);	// EEPROM.c
int SPI(void);													// SPI.c

//------------------------------------------------------------------ START
void get_rx_flag(void)
/*
	Creates a flag for incoming characters. Runs on a separate cog.
	Replaces the hardware "serial data ready" flag on other processors.
	
	INPUTS:		None
	RETURNS:	None
*/
	{
	rx_flag = -1;								// Initialise the flag to clear
	while(1)										// Continually check for a character on the USB port
		{
		global_key = getchar();		// Save the incoming char in a global memory location
		rx_flag = 0;							// Set the serial data ready flag
		while(rx_flag == 0);			// Wait until an external routine has read the data and cleared the flag
		}
	}
//------------------------------------------------------------------
void Flash(int pin)
/*
	Toggles a port pin to flash an LED or sound the buzzer.
	
	INPUTS:		int pin	=	pin number to flash
	RETURNS:	None
*/
	{
	int j;
	
	for (j = 0; j <= 4; j++)		// Flash 5 times
		{
		SET(OUTA,pin);						// Output high
		msleep(40);								// Wait
		CLR(OUTA,pin);						// Output low
		msleep(40);								// Wait
		}
	}	
//------------------------------------------------------------------
void Show_Strength(void)
/*
	Displays the signal strenght on a 0 - 9 scale using the LEDs.
	
	INPUTS:		None
	RETURNS:	None
*/
	{
	int j, str;
	
	str = (int)(signal_width);							// Convert the signal width in meters to an integer
																					// Check for limits
	if((Status & SIGNAL_VALID_MASK) == 0)		// If there is no signal use 0
		str = 0;
	if(str > 9)															// If strength > 9 then use 9
		str = 9;
		
	for (j = 0; j <= 9; j++)								// Set and clear all LEDs
		{
		if(j == str)													// Turn on the LED indicating signal strength
			SET(OUTA,j);
		else
			CLR(OUTA,j);												// Turn off all the others
		}
	}
//------------------------------------------------------------------
/*
	Routine to convert integer values from the DS00VQ100 chip to a floating point distance.
	
	INPUTS:		None
	RETURNS:	None
*/
float To_Dist (int value)													// Useful function used in this routine only
	{
	return((float)(value) * resolution);
	}

void Convert_to_Distance(void)
	{	
	int j;
	
	if(span_int == 0)																// Prevent divide-by-zero by forcing a resolution of 1 if there is no span result
		resolution = 1.0;
	else
		resolution = range / (float)(span_int);				// Otherwise calculate the resolution
	
																									// Calculate the minimum vallid signal width to compensate for changes in the span
	min_width = (int)((float)Min_Signal_Width / (resolution * 100));	
		
	zero_front = To_Dist(zero_front_int);						// Convert signals to distances
	signal_front = To_Dist(signal_front_int);
	signal_rear = To_Dist(signal_rear_int);	
	signal_width = To_Dist(signal_width_int);
	
																									// Apply corrections to the signal	
	j = (int)(signal_width);												// Firstly, compensate for signal width effects
	if((Status & SIGNAL_VALID_MASK) == 0)						// If the signal is lost, assume a width of zero
		j = 0;
	if(j > 15)																			// If the signal is more then 15m wide then limit it to 15m
		j = 15;
																									// Use linear interpolation to find a correction for the current width
	slope = slopes[j];															// Select a slope based on the integer width
	intercept = intercepts[j];											// Select an intercept based on the integer width
	correction = slope * signal_width + intercept;	// Calculate correction using y = mx + c
	
																									// Apply all corrections to calculate the distance
	distance = signal_front - zero_front - correction + ((float)zero_offset)/100;
// distance = front_of_signal - zero_pulse_offset - signal_width_correction	+ user_set_zero_datum_cm

	if(average_flag != 0)														// If the averaging flag is set then calculate a rolling average of 16 distances
		{
		if((Status & SIGNAL_VALID_MASK) != 0)					// Only average if the signal is valid
			{
			averages[average_pointer] = distance;				// Add a new distance to the averaging buffer
			average_pointer++;													// Point to the next buffer location ready for the next result
			}
		if(average_pointer > 15)											// Collect 16 answers in the buffer
			average_pointer = 0;
	
		distance = 0;																	// Average the values in the averaging buffer
		for(j = 0; j < 16; j++)
			distance = distance + averages[j];
		distance = distance / 16.0;
		}
				
	Show_Strength();																// Display the signal strength on the LEDs
	}

//------------------------------------------------------------------
/*
 Function to blink the LEDs and sound the buzzer five times.
 
 INPUTS:		None
 RETURNS:		None
*/
void BlinkLED(void)
	{
	int i, k;

	for(k = 0; k<=4; k++)
	  {
	  for (i = LED1; i <= BUZZER; i++)
			Flash(i);
	  for (i = BUZZER; i >= LED1; i--)
			Flash(i);
		}
	}
//------------------------------------------------------------------
/*
	Function to convert the resolution from meters into centimeters for display purposes.
	
	INPUTS:		None
	RETURNS:	float res	= floating point resolution in cm
*/
float Rescm(void)
	{
	float r;
	
	r = resolution * 100.0;				// Convert to centimeters
	if(direction == REVERSE)			// Use -ve sign to show reverse time expansion
		return(0.0 - r);
	return(r);
	}
//------------------------------------------------------------------
/*
	System hardware initialisation.
	Called once by main() on power up.
	
	INPUTS:		None
	RETURNS:	None
*/
void Initialise_Hardware(void)
	{
	long stack[8];
	int cog_id;
		
	clkset(0x6F, 5000000);																		// (0x6E, 4915200) = PLL x 8, (0x6F, 4915200) = PLL x 16;
	msleep(1000);																							// Wait for new clock settings to take effect
	asio_init(57600);																					// Initialise comms
	DIRA = DIRA | PORT_DIRECTIONS;														// Set port directions
	cog_id = cognew(get_rx_flag, &stack[7]);									// Start comms detection on separate cog
	SET(OUTA, CSNEEPROM);																			// Deselect the EERPOM on the timer board 
	}
//------------------------------------------------------------------	
/*
	Initialise software parameters.
	Called by main() on power up and the EEPROM editor when a new value is stored. 
	
	INPUTS:		None
	RETURNS:	None
*/	
void Initialise_Parameters(void)
	{
	int j;	
	
	zero_offset = Get_EEPROM_Float(EE_ZERO_OFFSET);						// Load the default values for the DS00VQ100 from the EEPROM
	Amplifier_Gain = Get_EEPROM_Int(EE_AMPLIFIER_GAIN);
	APD_Voltage = Get_EEPROM_Int(EE_APD_VOLTAGE);
	Measuring_Range = Get_EEPROM_Int(EE_MEASURING_RANGE);
	Min_Signal_Width = Get_EEPROM_Int(EE_SIGNAL_WIDTH);
	Test_Port = Get_EEPROM_Int(EE_TEST_PORT);
	Main_Xtal = Get_EEPROM_Int(EE_MAIN_XTAL);
	Second_Xtal = Get_EEPROM_Int(EE_SECOND_XTAL);
	
	serial_number = Get_EEPROM_Int(EE_SERIAL_NUMBER);				// Fetch the serial number of the LRF for display only
	min_width = Min_Signal_Width;														// Load a local variable for later use
	
	for(j = 0; j < 16; j++)																	// Fetch the signal width corrections and save locally
		corrections[j] = Get_EEPROM_Float(EE_CORRECTIONS + j);	

	for(j = 0; j < 15; j++)																	// Pre-calculate the signal width linearisation table for later use
		{
		slopes[j] = corrections[j+1] - corrections[j];
		intercepts[j] = corrections[j] - slopes[j] * (float)(j);
		}
	
	SPI_Footer = 0x77;																			// Set for immediate load to the DS00VQ100 chip
	}
//------------------------------------------------------------------ END





